package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.data.recipes.SimpleCookingRecipeBuilder
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraftforge.common.Tags
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import java.util.function.Consumer

fun addDecorativesRecipes(provider: MatteryRecipeProvider, consumer: Consumer<FinishedRecipe>) {
	// Напольная плитка
	for ((color, unrefinedItem) in MRegistry.UNREFINED_FLOOR_TILES.items) {
		MatteryRecipe(unrefinedItem, 24)
			.row(Items.CLAY, Items.SAND, Items.CLAY)
			.row(Items.SAND, color.tag, Items.SAND)
			.row(Items.CLAY, Items.SAND, Items.CLAY)
			.unlockedBy(Items.CLAY)
			.build(consumer)
	}

	for ((color, refinedItem) in MRegistry.FLOOR_TILES.items) {
		SimpleCookingRecipeBuilder.smelting(
			Ingredient.of(MRegistry.UNREFINED_FLOOR_TILES.items[color]!!),
			refinedItem,
			0.15f,
			200
		).unlockedBy("has_unrefined_block", has(MRegistry.UNREFINED_FLOOR_TILES.items[color]!!)).save(consumer)
	}

	for ((item, colors) in MRegistry.TRITANIUM_STRIPED_BLOCK.itemsWithColor) {
		val (base, stripe) = colors

		val original = MRegistry.TRITANIUM_BLOCK.items[base]!!

		MatteryRecipe(item, 8)
			.row(original, original, original)
			.row(original, stripe.tag, original)
			.row(original, original, original)
			.unlockedBy(original)
			.build(consumer)
	}

	MatteryRecipe(MRegistry.VENT.item, 24)
		.rowB(MItemTags.PLATE_TRITANIUM)
		.row(MItemTags.PLATE_TRITANIUM, Items.IRON_BARS, MItemTags.PLATE_TRITANIUM)
		.rowB(MItemTags.PLATE_TRITANIUM)
		.unlockedBy(MItemTags.PLATE_TRITANIUM)
		.build(consumer)

	for ((color, item) in MRegistry.VENT.items) {
		MatteryRecipe(item, 8)
			.row(MRegistry.VENT.item, MRegistry.VENT.item, MRegistry.VENT.item)
			.row(MRegistry.VENT.item, color.tag, MRegistry.VENT.item)
			.row(MRegistry.VENT.item, MRegistry.VENT.item, MRegistry.VENT.item)
			.unlockedBy(MItemTags.PLATE_TRITANIUM)
			.unlockedBy(color.tag)
			.build(consumer)
	}

	for ((color, item) in MRegistry.VENT_ALTERNATIVE.allItems) {
		val other = MRegistry.VENT.allItems[color]!!
		ShapelessRecipeBuilder(item, 1).requires(other).unlockedBy(item).save(consumer)
		ShapelessRecipeBuilder(other, 1).requires(item).unlockedBy(other).save(consumer, ResourceLocation(
			OverdriveThatMatters.MOD_ID, "${other.registryName!!.path}_from_alt"))
	}

	MatteryRecipe(MRegistry.DECORATIVE_CRATE.item, 24)
		.rowB(MItemTags.PLATE_IRON)
		.row(MItemTags.PLATE_IRON, Tags.Items.COBBLESTONE, MItemTags.PLATE_IRON)
		.rowB(MItemTags.PLATE_IRON)
		.unlockedBy(MItemTags.PLATE_IRON)
		.build(consumer)

	for ((color, crate) in MRegistry.DECORATIVE_CRATE.items) {
		MatteryRecipe(crate, 8)
			.row(MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item)
			.row(MRegistry.DECORATIVE_CRATE.item, color.tag, MRegistry.DECORATIVE_CRATE.item)
			.row(MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item)
			.unlockedBy(MRegistry.DECORATIVE_CRATE.item)
			.build(consumer)
	}

	for ((color, item) in MRegistry.TRITANIUM_BLOCK.items) {
		MatteryRecipe(item, 8)
			.row(MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item)
			.row(MRegistry.TRITANIUM_BLOCK.item, color.tag, MRegistry.TRITANIUM_BLOCK.item)
			.row(MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item)
			.unlockedBy(MRegistry.TRITANIUM_BLOCK.item)
			.unlockedBy(color.tag)
			.build(consumer)
	}

	ShapelessRecipeBuilder(MRegistry.INDUSTRIAL_GLASS.item, 8)
		.requires(Items.GLASS, 8)
		.requires(MItemTags.PLATE_TRITANIUM)
		.unlockedBy("has_plate", has(MItemTags.PLATE_TRITANIUM))
		.unlockedBy("has_glass", has(Items.GLASS))
		.save(consumer)

	ShapedRecipeBuilder(MRegistry.INDUSTRIAL_GLASS_PANE.item, 16)
		.define('#', MRegistry.INDUSTRIAL_GLASS.item)
		.pattern("###").pattern("###")
		.unlockedBy("has_tritanium_glass", has(MRegistry.INDUSTRIAL_GLASS.item))
		.save(consumer)

	val mappingUpgradeVanilla = mapOf(
		DyeColor.WHITE to Items.WHITE_STAINED_GLASS,
		DyeColor.ORANGE to Items.ORANGE_STAINED_GLASS,
		DyeColor.MAGENTA to Items.MAGENTA_STAINED_GLASS,
		DyeColor.LIGHT_BLUE to Items.LIGHT_BLUE_STAINED_GLASS,
		DyeColor.YELLOW to Items.YELLOW_STAINED_GLASS,
		DyeColor.LIME to Items.LIME_STAINED_GLASS,
		DyeColor.PINK to Items.PINK_STAINED_GLASS,
		DyeColor.GRAY to Items.GRAY_STAINED_GLASS,
		DyeColor.LIGHT_GRAY to Items.LIGHT_GRAY_STAINED_GLASS,
		DyeColor.CYAN to Items.CYAN_STAINED_GLASS,
		DyeColor.PURPLE to Items.PURPLE_STAINED_GLASS,
		DyeColor.BLUE to Items.BLUE_STAINED_GLASS,
		DyeColor.BROWN to Items.BROWN_STAINED_GLASS,
		DyeColor.GREEN to Items.GREEN_STAINED_GLASS,
		DyeColor.RED to Items.RED_STAINED_GLASS,
		DyeColor.BLACK to Items.BLACK_STAINED_GLASS,
	)

	for (color in DyeColor.values()) {
		val item = MRegistry.INDUSTRIAL_GLASS.items[color]!!
		val paneItem = MRegistry.INDUSTRIAL_GLASS_PANE.items[color]!!
		val mappedVanilla = mappingUpgradeVanilla[color]!!

		// обычная покраска
		ShapelessRecipeBuilder(item, 8)
			.requires(MRegistry.INDUSTRIAL_GLASS.item, 8)
			.requires(color.tag)
			.unlockedBy("has_tritanium_glass", has(MRegistry.INDUSTRIAL_GLASS.item))
			.save(consumer)

		// апгрейд ванильного крашенного стекла
		ShapelessRecipeBuilder(item, 8)
			.requires(mappedVanilla, 8)
			.requires(MItemTags.PLATE_TRITANIUM)
			.unlockedBy("has_plate", has(MItemTags.PLATE_TRITANIUM))
			.unlockedBy("has_colored_glass", has(mappedVanilla))
			.save(consumer, ResourceLocation(DataGen.MOD_ID, "${item.registryName!!.path}_upgrade"))

		ShapelessRecipeBuilder(paneItem, 8)
			.requires(MRegistry.INDUSTRIAL_GLASS_PANE.item, 8)
			.requires(color.tag)
			.unlockedBy("has_tritanium_glass_pane", has(MRegistry.INDUSTRIAL_GLASS_PANE.item))
			.save(consumer)

		ShapedRecipeBuilder(paneItem, 16)
			.define('#', item)
			.pattern("###").pattern("###")
			.unlockedBy("has_colored_tritanium_glass", has(paneItem))
			.save(consumer, ResourceLocation(DataGen.MOD_ID, "${paneItem.registryName!!.path}_alt"))
	}

	ShapelessRecipeBuilder(MItems.LABORATORY_LAMP, 1)
		.requires(MItems.LABORATORY_LAMP_INVERTED)
		.unlockedBy(MItems.LABORATORY_LAMP_INVERTED)
		.save(consumer, MItems.LABORATORY_LAMP.registryName!!.toString() + "_inv")

	ShapelessRecipeBuilder(MItems.LABORATORY_LAMP_INVERTED, 1)
		.requires(MItems.LABORATORY_LAMP)
		.unlockedBy(MItems.LABORATORY_LAMP)
		.save(consumer, MItems.LABORATORY_LAMP_INVERTED.registryName!!.toString() + "_inv")
}
