package ru.dbotthepony.mc.otm.client

import com.mojang.blaze3d.platform.InputConstants
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen
import net.minecraft.client.gui.screens.inventory.InventoryScreen
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.inventory.InventoryMenu
import net.minecraftforge.client.event.ClientPlayerNetworkEvent
import net.minecraftforge.client.event.MovementInputUpdateEvent
import net.minecraftforge.client.event.ScreenEvent
import net.minecraftforge.event.TickEvent
import net.minecraftforge.event.TickEvent.ClientTickEvent
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.capability.matteryPlayer
import ru.dbotthepony.mc.otm.client.render.UVWindingOrder
import ru.dbotthepony.mc.otm.client.render.Widgets18
import ru.dbotthepony.mc.otm.client.screen.ExoSuitInventoryScreen
import ru.dbotthepony.mc.otm.client.screen.panels.LargeRectangleButtonPanel
import ru.dbotthepony.mc.otm.client.screen.panels.Panel2Widget
import ru.dbotthepony.mc.otm.core.IConditionalTickable
import ru.dbotthepony.mc.otm.core.ITickable
import ru.dbotthepony.mc.otm.core.TickList
import ru.dbotthepony.mc.otm.core.TimerQueue
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.isClient
import ru.dbotthepony.mc.otm.registry.AndroidFeatures

fun onMovementInputUpdate(event: MovementInputUpdateEvent) {
	val ply = event.entity
	val input = event.input

	val cap = ply.matteryPlayer ?: return

	if (!cap.isAndroid || cap.hasFeature(AndroidFeatures.AIR_BAGS))
		return

	if (ply.abilities.mayfly) {
		cap.lastJumpTicks = 14
	} else {
		if (ply.isInWater) {
			if (ply.isOnGround) {
				cap.lastJumpTicks = 14
			}

			if (ply.isSwimming) {
				ply.isSwimming = false
			}

			if (cap.lastJumpTicks <= 0) {
				input.jumping = false
				input.up = false
			} else {
				cap.lastJumpTicks--
			}
		}
	}
}

var shouldOpenVanillaInventory = false

fun onPostScreenInit(event: ScreenEvent.Init.Post) {
	val player = minecraft.player?.matteryPlayer ?: return

	val eventScreen = event.screen
	val screen = if (eventScreen is AbstractContainerScreen<*> && eventScreen.menu is InventoryMenu) eventScreen else return

	if (player.hasExoSuit) {
		val widget = Panel2Widget(LargeRectangleButtonPanel(screen, null,
			x = screen.guiLeft + screen.xSize - 2f - LargeRectangleButtonPanel.SIZE,
			y = screen.guiTop.toFloat() - LargeRectangleButtonPanel.SIZE - 2,
			skinElement = Widgets18.RETURN_ARROW_LEFT,
			skinElementWinding = UVWindingOrder.FLOP,
			onPress = {
				shouldOpenVanillaInventory = false
				val mouseX = minecraft.mouseHandler.xpos()
				val mouseY = minecraft.mouseHandler.ypos()

				event.screen.onClose()
				minecraft.setScreen(ExoSuitInventoryScreen(player.exoSuitMenu))

				InputConstants.grabOrReleaseMouse(minecraft.window.window, 212993, mouseX, mouseY)
			}).also { it.tooltip = TranslatableComponent("otm.gui.exosuit.go_in") })

		event.addListener(widget)

		tickWhileClient({ minecraft.screen == screen }) {
			widget.panel.x = screen.guiLeft + screen.xSize - 2f - LargeRectangleButtonPanel.SIZE
			widget.panel.y = screen.guiTop.toFloat() - LargeRectangleButtonPanel.SIZE - 2f
		}
	}
}

fun onScreenOpen(event: ScreenEvent.Opening) {
	if (shouldOpenVanillaInventory || minecraft.player?.isCreative == true) {
		return
	}

	val player = minecraft.player?.matteryPlayer ?: return

	if (player.hasExoSuit && event.newScreen is InventoryScreen) {
		event.newScreen = ExoSuitInventoryScreen(player.exoSuitMenu)
	}
}

private val preTickList = TickList()
private val postTickList = TickList()

private val preTimerList = TimerQueue()
private val postTimerList = TimerQueue()

private var LOGGED_IN = false
private val LOGGER = LogManager.getLogger()

fun onceClient(ticker: ITickable) {
	check(isClient) { "Illegal side" }
	postTickList.add(ticker, LOGGED_IN, "Not logged in")
}

fun onceClientPre(ticker: ITickable) {
	check(isClient) { "Illegal side" }
	preTickList.add(ticker, LOGGED_IN, "Not logged in")
}

fun tickClient(ticker: IConditionalTickable) {
	check(isClient) { "Illegal side" }
	postTickList.add(ticker, LOGGED_IN, "Not logged in")
}

fun tickClientPre(ticker: IConditionalTickable) {
	check(isClient) { "Illegal side" }
	preTickList.add(ticker, LOGGED_IN, "Not logged in")
}

fun tickUntilClient(ticker: () -> Boolean) {
	tickClient(IConditionalTickable.wrap(ticker))
}

fun tickUntilClientPre(ticker: () -> Boolean) {
	tickClientPre(IConditionalTickable.wrap(ticker))
}

fun tickWhileClient(condition: () -> Boolean, ticker: () -> Unit) {
	tickClient(IConditionalTickable.wrap(condition, ticker))
}

fun tickWhileClientPre(condition: () -> Boolean, ticker: () -> Unit) {
	tickClientPre(IConditionalTickable.wrap(condition, ticker))
}

fun onClientTick(event: ClientTickEvent) {
	if (event.phase == TickEvent.Phase.START) {
		preTimerList.tick()
		preTickList.tick()
	} else {
		postTimerList.tick()
		postTickList.tick()
	}
}

fun onClientDisconnected(event: ClientPlayerNetworkEvent.LoggingOut) {
	LOGGED_IN = false

	preTimerList.clear()
	preTickList.clear()
	postTimerList.clear()
	postTickList.clear()
}

fun onClientConnected(event: ClientPlayerNetworkEvent.LoggingIn) {
	LOGGED_IN = true

	preTimerList.clear()
	preTickList.clear()
	postTimerList.clear()
	postTickList.clear()
}

