package ru.dbotthepony.mc.otm.client

import net.minecraft.client.Minecraft
import net.minecraft.client.gui.Font
import net.minecraft.client.resources.sounds.SimpleSoundInstance
import net.minecraft.sounds.SoundEvents
import org.lwjgl.glfw.GLFW
import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.nio.DoubleBuffer

inline val minecraft: Minecraft get() = Minecraft.getInstance()
inline val font: Font get() = minecraft.font

fun playGuiClickSound() {
	minecraft.soundManager.play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0f))
}

fun setMousePos(x: Double, y: Double) {
	GLFW.glfwSetCursorPos(minecraft.window.window, x, y)
}

fun setMousePosScaled(x: Double, y: Double) {
	GLFW.glfwSetCursorPos(minecraft.window.window, x * minecraft.window.guiScale, y * minecraft.window.guiScale)
}

fun setMousePos(x: Float, y: Float) {
	GLFW.glfwSetCursorPos(minecraft.window.window, x.toDouble(), y.toDouble())
}

fun setMousePosScaled(x: Float, y: Float) {
	GLFW.glfwSetCursorPos(minecraft.window.window, x * minecraft.window.guiScale, y * minecraft.window.guiScale)
}

private val cursorXPosBuf = ByteBuffer.allocateDirect(8).also { it.order(ByteOrder.LITTLE_ENDIAN) }.asDoubleBuffer()
private val cursorYPosBuf = ByteBuffer.allocateDirect(8).also { it.order(ByteOrder.LITTLE_ENDIAN) }.asDoubleBuffer()

data class MousePos(val x: Double, val y: Double) {
	fun set() {
		setMousePos(x, y)
	}

	fun move(x: Double = 0.0, y: Double = 0.0) {
		GLFW.glfwSetCursorPos(minecraft.window.window, this.x + x, this.y + y)
	}

	fun move(x: Float = 0.0f, y: Float = 0.0f) {
		GLFW.glfwSetCursorPos(minecraft.window.window, this.x + x, this.y + y)
	}

	fun moveScaled(x: Double = 0.0, y: Double = 0.0) {
		GLFW.glfwSetCursorPos(minecraft.window.window, this.x + x * minecraft.window.guiScale, this.y + y * minecraft.window.guiScale)
	}

	fun moveScaled(x: Float = 0.0f, y: Float = 0.0f) {
		GLFW.glfwSetCursorPos(minecraft.window.window, this.x + x * minecraft.window.guiScale, this.y + y * minecraft.window.guiScale)
	}
}

val mousePos: MousePos get() {
	cursorXPosBuf.position(0)
	cursorYPosBuf.position(0)

	GLFW.glfwGetCursorPos(minecraft.window.window, cursorXPosBuf, cursorYPosBuf)

	cursorXPosBuf.position(0)
	cursorYPosBuf.position(0)

	return MousePos(cursorXPosBuf.get(), cursorYPosBuf.get())
}

fun moveMousePos(x: Double = 0.0, y: Double = 0.0) {
	val (currentX, currentY) = mousePos
	GLFW.glfwSetCursorPos(minecraft.window.window, currentX + x, currentY + y)
}

fun moveMousePos(x: Float = 0.0f, y: Float = 0.0f) {
	val (currentX, currentY) = mousePos
	GLFW.glfwSetCursorPos(minecraft.window.window, currentX + x, currentY + y)
}

fun moveMousePosScaled(x: Double = 0.0, y: Double = 0.0) {
	val (currentX, currentY) = mousePos
	GLFW.glfwSetCursorPos(minecraft.window.window, currentX + x * minecraft.window.guiScale, currentY + y * minecraft.window.guiScale)
}

fun moveMousePosScaled(x: Float = 0.0f, y: Float = 0.0f) {
	val (currentX, currentY) = mousePos
	GLFW.glfwSetCursorPos(minecraft.window.window, currentX + x * minecraft.window.guiScale, currentY + y * minecraft.window.guiScale)
}
