package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.renderer.GameRenderer
import net.minecraft.network.chat.Component
import net.minecraft.world.item.ItemStack
import net.minecraftforge.client.extensions.common.IClientItemExtensions
import org.lwjgl.opengl.GL11
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.core.RGBAColor

abstract class AbstractSlotPanel<out S : MatteryScreen<*>> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	open val noItemIcon: SkinElement? = null
) : EditablePanel<S>(screen, parent, x, y, width, height), IItemStackPanel {
	protected open fun renderSlotBackground(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		SLOT_BACKGROUND.render(stack, width = width, height = height)
	}

	protected open fun renderRegular(stack: PoseStack, itemstack: ItemStack, count_override: String? = null) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader)

		if (!itemstack.isEmpty) {
			RenderSystem.enableDepthTest()

			val systemPoseStack = RenderSystem.getModelViewStack()

			systemPoseStack.pushPose()
			systemPoseStack.translate((absoluteX + 1f).toDouble(), (absoluteY + 1f).toDouble(), 0.0)
			RenderSystem.applyModelViewMatrix()
			RenderSystem.depthFunc(GL11.GL_LESS)

			// Thanks Mojang
			// Very cool
			// (for int x, int y, which are then cast into doubles anyway)
			screen.itemRenderer.blitOffset = 1f // Z pos

			screen.itemRenderer.renderAndDecorateItem(
				requireNotNull(minecraft.player) { "yo, dude, what the fuck" },
				itemstack,
				0,
				0,
				(absoluteX + absoluteY * 1000f).toInt()
			)

			RenderSystem.depthFunc(GL11.GL_ALWAYS)
			screen.itemRenderer.renderGuiItemDecorations(screen.font, itemstack, 0, 0, count_override)
			screen.itemRenderer.blitOffset = 0f

			// too big accumulations can lead to Z near clipping issues
			systemPoseStack.popPose()
			RenderSystem.applyModelViewMatrix()

			clearDepth(stack)
		}

		if (isHovered) {
			drawColor = SLOT_HIGHLIGHT
			drawRect(stack, 1f, 1f, width - 1, height - 1)
		}
	}

	protected open fun getItemStackTooltip(stack: ItemStack): List<Component> {
		return screen.getTooltipFromItem(stack)
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		renderSlotBackground(stack, mouseX, mouseY, partialTick)
		val itemStack = itemStack
		renderRegular(stack, itemStack)

		if (itemStack.isEmpty) {
			noItemIcon?.render(stack, width = width, height = height)
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float): Boolean {
		if (isHovered) {
			val itemstack = itemStack

			if (!itemstack.isEmpty) {
				// val font = RenderProperties.get(itemstack).getFont(itemstack)
				val font = (itemstack.item.renderPropertiesInternal as? IClientItemExtensions)?.getFont(itemstack, IClientItemExtensions.FontContext.TOOLTIP)

				// TODO: WHERE????????????
				// GuiUtils.preItemToolTip(itemstack);
				screen.renderComponentTooltip(
					stack,
					getItemStackTooltip(itemstack),
					mouseX.toInt(),
					mouseY.toInt(),
					font ?: screen.font,
					itemstack
				)
				// GuiUtils.postItemToolTip();

				return true
			}
		}
		return false
	}

	companion object {
		val SLOT_HIGHLIGHT = RGBAColor(255, 255, 255, 100)
		val SLOT_HIGHLIGHT_DRAG = RGBAColor(200, 200, 200, 150)
		const val SIZE = 18f
		val SLOT_BACKGROUND = WidgetLocation.WIDGETS.element(0f, 0f, SIZE, SIZE)
	}
}
