package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.platform.InputConstants
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.ChatFormatting
import net.minecraft.client.gui.screens.Screen
import net.minecraft.network.chat.Component
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.core.RGBAColor
import ru.dbotthepony.mc.otm.core.next
import ru.dbotthepony.mc.otm.core.prev
import java.util.*
import kotlin.collections.ArrayList
import kotlin.reflect.KMutableProperty0

open class ButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = 40f,
	height: Float = HEIGHT,
	var label: Component
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	constructor(screen: S, parent: EditablePanel<*>?, label: Component) : this(screen, parent, x = 0f, label = label)

	constructor(
		screen: S,
		parent: EditablePanel<*>?,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = 40f,
		height: Float = HEIGHT,
		label: Component,
		onPress: Runnable
	) : this(screen, parent, x, y, width, height, label) {
		this.callback = onPress
	}

	protected var callback: Runnable? = null
	protected var pressed = false

	protected open fun onClick(button: Int) {
		callback?.run()
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (isDisabled || pressed) {
			return true
		}

		pressed = true
		trapMouseInput = true
		playGuiClickSound()
		return true
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (!pressed) {
			return true
		}

		trapMouseInput = false
		pressed = false

		if (isHovered) {
			onClick(button)
		}

		return true
	}

	fun bind(runnable: Runnable) {
		callback = runnable
	}

	var textColor = RGBAColor.WHITE

	var isDisabled = false
		set(value) {
			if (field != value) {
				if (!value) {
					pressed = false
					trapMouseInput = false
				}

				field = value
			}
		}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (isDisabled) {
			Widgets18.BUTTON_DISABLED_STRETCHABLE.render(stack, width = width, height = height)
		} else if (pressed) {
			Widgets18.BUTTON_PRESSED_STRETCHABLE.render(stack, width = width, height = height)
		} else if (isHovered) {
			Widgets18.BUTTON_HOVERED_STRETCHABLE.render(stack, width = width, height = height)
		} else {
			Widgets18.BUTTON_IDLE_STRETCHABLE.render(stack, width = width, height = height)
		}

		font.drawAligned(stack, label, TextAlign.CENTER_CENTER, width / 2f, height / 2f, textColor.toInt())
	}

	override fun sizeToContents() {
		super.sizeToContents()

		height = height.coerceAtLeast(HEIGHT).coerceAtLeast(font.lineHeight.toFloat() + 2f)
		width = width.coerceAtLeast(HEIGHT).coerceAtLeast(font.width(label) + 4f)
	}

	companion object {
		const val HEIGHT = 18f
	}
}

@Suppress("PropertyName")
abstract class RectangleButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float,
	height: Float,
	val onPress: ((clickButton: Int) -> Unit)? = null,
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	protected var pressed = false

	protected open fun onClick(clickButton: Int) {
		onPress?.invoke(clickButton)
	}

	abstract val PRESSED: SkinElement
	abstract val HOVERED: SkinElement
	abstract val IDLE: SkinElement
	abstract val DISABLED: SkinElement

	var isDisabled = false
		set(value) {
			if (field != value) {
				if (!value) {
					pressed = false
					trapMouseInput = false
				}

				field = value
			}
		}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (isDisabled) {
			DISABLED.render(stack, 0f, 0f, width, height)
		} else if (pressed) {
			PRESSED.render(stack, 0f, 0f, width, height)
		} else if (isHovered) {
			HOVERED.render(stack, 0f, 0f, width, height)
		} else {
			IDLE.render(stack, 0f, 0f, width, height)
		}
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled) {
			if (!pressed) {
				playGuiClickSound()
			}

			pressed = true
			trapMouseInput = true
		}

		return true
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled && pressed) {
			pressed = false

			if (isHovered) {
				onClick(button)
			}
		}

		trapMouseInput = false

		return true
	}
}

abstract class EnumRectangleButtonPanel<out S : Screen, T : Enum<T>>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float,
	height: Float,
	val enum: Class<T>,
	val prop: KMutableProperty0<T>,
	val defaultValue: T,
	val onChange: ((newValue: T) -> Unit)? = null,
) : RectangleButtonPanel<S>(screen, parent, x, y, width, height, null) {
	private var building = true

	protected val enumMapping = EnumMap<T, Pair<SkinElement, UVWindingOrder>>(enum)
	protected val tooltipMapping = EnumMap<T, Component>(enum)

	fun addTooltip(value: T, component: Component): EnumRectangleButtonPanel<S, T> {
		check(tooltipMapping.put(value, component) == null) { "Already has mapping for $value" }
		return this
	}

	var mainTooltip: Component? = null
		set(value) {
			if (field != null) {
				throw UnsupportedOperationException("Write once only")
			}

			field = value
		}

	fun isFullyDefined(): Boolean {
		for (value in enum.enumConstants) {
			if (enumMapping[value] == null) {
				return false
			}
		}

		return true
	}

	val missingValues: List<T> get() {
		val missing = ArrayList<T>()

		for (value in enum.enumConstants) {
			if (enumMapping[value] == null) {
				missing.add(value)
			}
		}

		return missing
	}

	fun add(value: T, skinElement: SkinElement, winding: UVWindingOrder = UVWindingOrder.NORMAL): EnumRectangleButtonPanel<S, T> {
		return add(value, skinElement to winding)
	}

	fun add(value: T, skinElement: SkinElement, component: Component, winding: UVWindingOrder = UVWindingOrder.NORMAL): EnumRectangleButtonPanel<S, T> {
		return add(value, component, skinElement to winding)
	}

	fun add(value: T, component: Component, skinElement: SkinElement, winding: UVWindingOrder = UVWindingOrder.NORMAL): EnumRectangleButtonPanel<S, T> {
		return add(value, component, skinElement to winding)
	}

	fun add(value: T, pair: Pair<SkinElement, UVWindingOrder>): EnumRectangleButtonPanel<S, T> {
		check(building) { "Not building" }
		check(enumMapping.put(value, pair) == null) { "Already has mapping for $value" }

		if (enumMapping.size == enum.enumConstants.size) {
			finish()
		}

		return this
	}

	fun add(value: T, component: Component, pair: Pair<SkinElement, UVWindingOrder>): EnumRectangleButtonPanel<S, T> {
		addTooltip(value, component)
		return add(value, pair)
	}

	fun finish(): EnumRectangleButtonPanel<S, T> {
		check(building) { "Not building" }
		check(isFullyDefined()) {
			"Not all enums having their mapping defined, missing are: ${missingValues.joinToString(", ")}"
		}

		building = false
		return this
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		check(!building) { "Still building this button!" }
		super.innerRender(stack, mouseX, mouseY, partialTick)
		val pair = checkNotNull(enumMapping[prop.get()]) { "HOW" }
		pair.first.render(stack, 0f, 0f, width, height, pair.second)
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		check(!building) { "Still building this button!" }
		return super.mouseClickedInner(x, y, button)
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		check(!building) { "Still building this button!" }
		return super.mouseReleasedInner(x, y, button)
	}

	override fun onClick(clickButton: Int) {
		when (clickButton) {
			InputConstants.MOUSE_BUTTON_LEFT -> {
				prop.set(prop.get().next(enum.enumConstants))
				onChange?.invoke(prop.get())
			}

			InputConstants.MOUSE_BUTTON_RIGHT -> {
				prop.set(prop.get().prev(enum.enumConstants))
				onChange?.invoke(prop.get())
			}

			InputConstants.MOUSE_BUTTON_MIDDLE -> {
				if (prop.get() != defaultValue) {
					prop.set(defaultValue)
					onChange?.invoke(prop.get())
				}
			}
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float): Boolean {
		if (!isHovered && !isTrappingMouseInput()) {
			return super.innerRenderTooltips(stack, mouseX, mouseY, partialTick)
		}

		if (mainTooltip == null && tooltipMapping.size == 0) {
			return super.innerRenderTooltips(stack, mouseX, mouseY, partialTick)
		}

		val listing = ArrayList<Component>()

		if (mainTooltip != null) {
			listing.add(mainTooltip!!)
			listing.add(SPACE)
		}

		for ((key, value) in tooltipMapping) {
			if (key == prop.get()) {
				listing.add(value.copy().withStyle(ChatFormatting.WHITE))
			} else {
				listing.add(value.copy().withStyle(ChatFormatting.GRAY))
			}
		}

		screen.renderComponentTooltip(
			stack,
			listing,
			mouseX.toInt(),
			mouseY.toInt(),
			font
		)

		return true
	}

	companion object {
		private val SPACE = TextComponent("")
	}
}

open class LargeRectangleButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	onPress: ((clickButton: Int) -> Unit)? = null,
	val skinElement: SkinElement? = null,
	val skinElementWinding: UVWindingOrder? = null,
) : RectangleButtonPanel<S>(screen, parent, x, y, width, height, onPress) {
	final override val IDLE = Widgets18.BUTTON_IDLE
	final override val HOVERED = Widgets18.BUTTON_HOVERED
	final override val PRESSED = Widgets18.BUTTON_PRESSED
	final override val DISABLED = Widgets18.BUTTON_DISABLED

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		super.innerRender(stack, mouseX, mouseY, partialTick)

		if (skinElementWinding != null) {
			skinElement?.render(stack, width = width, height = height, winding = skinElementWinding)
		} else {
			skinElement?.render(stack, width = width, height = height)
		}
	}

	companion object {
		const val SIZE = 18f
	}
}

open class LargeEnumRectangleButtonPanel<out S : Screen, T : Enum<T>>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	enum: Class<T>,
	prop: KMutableProperty0<T>,
	defaultValue: T,
	onChange: ((newValue: T) -> Unit)? = null,
) : EnumRectangleButtonPanel<S, T>(screen, parent, x, y, width, height, enum, prop, defaultValue, onChange) {
	final override val IDLE = Widgets18.BUTTON_IDLE
	final override val HOVERED = Widgets18.BUTTON_HOVERED
	final override val PRESSED = Widgets18.BUTTON_PRESSED
	final override val DISABLED = Widgets18.BUTTON_DISABLED

	companion object {
		const val SIZE = 18f
	}
}

open class SmallRectangleButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	onPress: ((clickButton: Int) -> Unit)? = null,
	val skinElement: SkinElement? = null,
	val skinElementWinding: UVWindingOrder? = null,
) : RectangleButtonPanel<S>(screen, parent, x, y, width, height, onPress) {
	final override val IDLE = Widgets8.BUTTON_IDLE
	final override val HOVERED = Widgets8.BUTTON_HOVERED
	final override val PRESSED = Widgets8.BUTTON_PRESSED
	final override val DISABLED = Widgets8.BUTTON_DISABLED

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		super.innerRender(stack, mouseX, mouseY, partialTick)

		if (skinElementWinding != null) {
			skinElement?.render(stack, width = width, height = height, winding = skinElementWinding)
		} else {
			skinElement?.render(stack, width = width, height = height)
		}
	}

	companion object {
		const val SIZE = 8f
	}
}

open class SmallEnumRectangleButtonPanel<out S : Screen, T : Enum<T>>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	enum: Class<T>,
	prop: KMutableProperty0<T>,
	defaultValue: T,
	onChange: ((newValue: T) -> Unit)? = null,
) : EnumRectangleButtonPanel<S, T>(screen, parent, x, y, width, height, enum, prop, defaultValue, onChange) {
	final override val IDLE = Widgets8.BUTTON_IDLE
	final override val HOVERED = Widgets8.BUTTON_HOVERED
	final override val PRESSED = Widgets8.BUTTON_PRESSED
	final override val DISABLED = Widgets8.BUTTON_DISABLED

	companion object {
		const val SIZE = 8f
	}
}
