package ru.dbotthepony.mc.otm.compat.curios

import it.unimi.dsi.fastutil.ints.Int2ObjectArrayMap
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.Slot
import net.minecraftforge.fml.ModList
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.core.orNull
import top.theillusivec4.curios.api.CuriosApi
import top.theillusivec4.curios.common.inventory.CosmeticCurioSlot
import top.theillusivec4.curios.common.inventory.CurioSlot

val isCuriosLoaded by lazy {
	ModList.get().isLoaded(CuriosApi.MODID)
}

private fun Player.getCuriosSlotsImpl(): Collection<Pair<Slot, Slot?>> {
	val handler = getCapability(MatteryCapability.CURIOS_INVENTORY).orNull() ?: return listOf()

	val result = ArrayList<Pair<Slot, Slot?>>()

	for ((identifier, curio) in handler.curios) {
		if (curio.isVisible) {
			val stacks = curio.stacks

			for (slot in 0 until stacks.slots) {
				val regular = CurioSlot(this, stacks, slot, identifier, 0, 0, curio.renders)

				if (curio.hasCosmetic()) {
					val cosmetic = CosmeticCurioSlot(this, curio.cosmeticStacks, slot, identifier, 0, 0)

					result.add(regular to cosmetic)
				} else {
					result.add(regular to null)
				}
			}
		}
	}

	return result
}

/**
 * [Pair]<Regular, Cosmetic>
 */
val Player.curiosSlots: Collection<Pair<Slot, Slot?>> get() {
	if (!isCuriosLoaded) {
		return listOf()
	}

	return getCuriosSlotsImpl()
}
