package ru.dbotthepony.mc.otm.core

import net.minecraft.nbt.ByteArrayTag
import net.minecraft.nbt.ByteTag
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.DoubleTag
import net.minecraft.nbt.FloatTag
import net.minecraft.nbt.IntArrayTag
import net.minecraft.nbt.IntTag
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.LongArrayTag
import net.minecraft.nbt.LongTag
import net.minecraft.nbt.ShortTag
import net.minecraft.nbt.StringTag
import net.minecraft.nbt.Tag
import net.minecraft.world.item.ItemStack
import java.util.UUID

operator fun CompoundTag.set(index: String, value: Tag) = 				put(index, value)
operator fun CompoundTag.set(index: String, value: Int) = 				putInt(index, value)
operator fun CompoundTag.set(index: String, value: Byte) = 				putByte(index, value)
operator fun CompoundTag.set(index: String, value: Short) = 			putShort(index, value)
operator fun CompoundTag.set(index: String, value: Long) = 				putLong(index, value)
operator fun CompoundTag.set(index: String, value: Float) = 			putFloat(index, value)
operator fun CompoundTag.set(index: String, value: Double) = 			putDouble(index, value)
operator fun CompoundTag.set(index: String, value: String) = 			putString(index, value)
operator fun CompoundTag.set(index: String, value: Boolean) = 			putBoolean(index, value)
operator fun CompoundTag.set(index: String, value: ByteArray) = 		putByteArray(index, value)
operator fun CompoundTag.set(index: String, value: IntArray) = 			putIntArray(index, value)
operator fun CompoundTag.set(index: String, value: LongArray) = 		putLongArray(index, value)

operator fun CompoundTag.set(index: String, value: UUID) = 				putUUID(index, value)

fun CompoundTag.contains(vararg keys: String): Boolean {
	for (key in keys) {
		if (!contains(key)) {
			return false
		}
	}

	return true
}

inline fun <R, reified T : Tag> CompoundTag.map(key: String, consumer: (T) -> R): R? {
	val tag = get(key)

	if (tag is T) {
		return consumer(tag)
	}

	return null
}

inline fun <reified R : Enum<R>> CompoundTag.getEnum(key: String): R {
	val tag = get(key)

	if (tag is StringTag) {
		return R::class.java.enumConstants.first { it.name == tag.asString }
	}

	return R::class.java.enumConstants[0]
}

fun CompoundTag.getItemStack(key: String): ItemStack = map(key, ItemStack::of) ?: ItemStack.EMPTY

inline fun CompoundTag.ifHas(s: String, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null) {
		consumer(tag)
	}
}

inline fun CompoundTag.ifHas(s: String, type: Byte, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null && tag.id == type) {
		consumer(tag)
	}
}

inline fun <reified T : Tag> CompoundTag.ifHas(s: String, type: Class<T>, consumer: (T) -> Unit) {
	val tag = get(s)

	if (tag != null && tag::class.java === type) {
		consumer(tag as T)
	}
}

fun CompoundTag.getList(key: String): ListTag {
	return this[key] as? ListTag ?: ListTag()
}

fun CompoundTag.getByteList(key: String): MutableList<ByteTag> = getList(key, Tag.TAG_BYTE.toInt()) as MutableList<ByteTag>
fun CompoundTag.getShortList(key: String): MutableList<ShortTag> = getList(key, Tag.TAG_SHORT.toInt()) as MutableList<ShortTag>
fun CompoundTag.getIntList(key: String): MutableList<IntTag> = getList(key, Tag.TAG_INT.toInt()) as MutableList<IntTag>
fun CompoundTag.getLongList(key: String): MutableList<LongTag> = getList(key, Tag.TAG_LONG.toInt()) as MutableList<LongTag>
fun CompoundTag.getFloatList(key: String): MutableList<FloatTag> = getList(key, Tag.TAG_FLOAT.toInt()) as MutableList<FloatTag>
fun CompoundTag.getDoubleList(key: String): MutableList<DoubleTag> = getList(key, Tag.TAG_DOUBLE.toInt()) as MutableList<DoubleTag>
fun CompoundTag.getByteArrayList(key: String): MutableList<ByteArrayTag> = getList(key, Tag.TAG_BYTE_ARRAY.toInt()) as MutableList<ByteArrayTag>
fun CompoundTag.getStringList(key: String): MutableList<StringTag> = getList(key, Tag.TAG_STRING.toInt()) as MutableList<StringTag>
fun CompoundTag.getListList(key: String): MutableList<ListTag> = getList(key, Tag.TAG_LIST.toInt()) as MutableList<ListTag>
fun CompoundTag.getCompoundList(key: String): MutableList<CompoundTag> = getList(key, Tag.TAG_COMPOUND.toInt()) as MutableList<CompoundTag>
fun CompoundTag.getIntArrayList(key: String): MutableList<IntArrayTag> = getList(key, Tag.TAG_INT_ARRAY.toInt()) as MutableList<IntArrayTag>
fun CompoundTag.getLongArrayList(key: String): MutableList<LongArrayTag> = getList(key, Tag.TAG_LONG_ARRAY.toInt()) as MutableList<LongArrayTag>
