package ru.dbotthepony.mc.otm.recipe

import com.google.gson.JsonObject
import net.minecraft.network.FriendlyByteBuf
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.inventory.CraftingContainer
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.crafting.CraftingRecipe
import net.minecraft.world.item.crafting.RecipeSerializer
import net.minecraft.world.item.crafting.ShapedRecipe
import net.minecraft.world.level.Level
import ru.dbotthepony.mc.otm.capability.ItemEnergyStorageImpl
import ru.dbotthepony.mc.otm.capability.matteryEnergy
import ru.dbotthepony.mc.otm.container.stream
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.core.tagNotNull

class EnergyContainerRecipe(
	val parent: ShapedRecipe
) : CraftingRecipe by parent {
	override fun assemble(container: CraftingContainer): ItemStack {
		val itemStack = parent.assemble(container)

		val battery = container.stream()
			.filter { !it.isEmpty }
			.map { it.matteryEnergy }
			.filter { it != null }
			.findAny().orElse(null)

		if (battery != null) {
			itemStack.tagNotNull[ItemEnergyStorageImpl.ENERGY_KEY] = battery.batteryLevel.serializeNBT()
		}

		return itemStack
	}

	override fun matches(p_44002_: CraftingContainer, p_44003_: Level): Boolean {
		return parent.matches(p_44002_, p_44003_) && !p_44002_.stream().anyMatch { it.isDamaged }
	}

	override fun getSerializer(): RecipeSerializer<*> {
		return Companion
	}

	companion object : RecipeSerializer<EnergyContainerRecipe> {
		override fun fromJson(p_44103_: ResourceLocation, p_44104_: JsonObject): EnergyContainerRecipe {
			return EnergyContainerRecipe(ShapedRecipe.Serializer.SHAPED_RECIPE.fromJson(p_44103_, p_44104_))
		}

		override fun fromNetwork(p_44105_: ResourceLocation, p_44106_: FriendlyByteBuf): EnergyContainerRecipe? {
			return ShapedRecipe.Serializer.SHAPED_RECIPE.fromNetwork(p_44105_, p_44106_)?.let(::EnergyContainerRecipe)
		}

		override fun toNetwork(p_44101_: FriendlyByteBuf, p_44102_: EnergyContainerRecipe) {
			ShapedRecipe.Serializer.SHAPED_RECIPE.toNetwork(p_44101_, p_44102_.parent)
		}
	}

}
